@@ -16,12 +16,11 @@ use crate::{
     writer::Writer,
     BytesMut, JsonValue, ParamMap,
 };
-use atelier_core::model::shapes::ShapeKind;
 use atelier_core::{
     model::{
         shapes::{
             AppliedTraits, HasTraits, ListOrSet, Map as MapShape, MemberShape, Operation, Service,
-            Simple, StructureOrUnion,
+            ShapeKind, Simple, StructureOrUnion,
         },
         values::Value,
         HasIdentity, Identifier, Model, NamespaceID, ShapeID,
@@ -38,7 +37,6 @@ use atelier_core::{
 use std::{collections::HashMap, path::Path, str::FromStr, string::ToString};
 
 const WASMBUS_RPC_CRATE: &str = "wasmbus_rpc";
-
 const DEFAULT_MAP_TYPE: &str = "std::collections::HashMap";
 const DEFAULT_LIST_TYPE: &str = "Vec";
 const DEFAULT_SET_TYPE: &str = "std::collections::BTreeSet";
@@ -51,7 +49,6 @@ struct Declaration(u8, BytesMut);
 
 type ShapeList<'model> = Vec<(&'model ShapeID, &'model AppliedTraits, &'model ShapeKind)>;
 
-#[derive(Default)]
 pub struct RustCodeGen<'model> {
     /// if set, limits declaration output to this namespace only
     pub(crate) namespace: Option<NamespaceID>,
@@ -71,6 +68,24 @@ impl<'model> RustCodeGen<'model> {
     }
 }
 
+struct ServiceInfo<'model> {
+    id: &'model Identifier,
+    traits: &'model AppliedTraits,
+    service: &'model Service,
+}
+
+impl<'model> ServiceInfo<'model> {
+    fn wasmbus_contract_id(&self) -> Option<String> {
+        match get_trait(self.traits, crate::model::wasmbus_trait()) {
+            Ok(Some(Wasmbus {
+                contract_id: Some(contract_id),
+                ..
+            })) => Some(contract_id),
+            _ => None,
+        }
+    }
+}
+
 #[non_exhaustive]
 enum MethodArgFlags {
     Normal,
@@ -82,6 +97,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
     fn output_language(&self) -> OutputLanguage {
         OutputLanguage::Rust
     }
+
     /// Initialize code generator and renderer for language output.j
     /// This hook is called before any code is generated and can be used to initialize code generator
     /// and/or perform additional processing before output files are created.
@@ -96,9 +112,52 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         self.import_core = WASMBUS_RPC_CRATE.to_string();
 
         if let Some(model) = model {
+            if let Some(Value::Array(codegen_min)) = model.metadata_value("codegen") {
+                let current_ver =
+                    semver::Version::parse(env!("CARGO_PKG_VERSION")).map_err(|e| {
+                        Error::InvalidModel(format!(
+                            "parse error for weld-codegen package version: {}",
+                            e
+                        ))
+                    })?;
+                for val in codegen_min.iter() {
+                    if let Value::Object(map) = val {
+                        if let Some(Value::String(lang)) = map.get("language") {
+                            if lang.as_str() == "rust" {
+                                if let Some(Value::String(ver)) = map.get("min_version") {
+                                    let min_ver = semver::Version::parse(ver).map_err(|e| {
+                                        Error::InvalidModel(format!(
+                                            "metadata parse error for codegen {{ language=rust, \
+                                             min_version={} }}: {}",
+                                            ver, e
+                                        ))
+                                    })?;
+                                    if min_ver.gt(&current_ver) {
+                                        return Err(Error::Model(format!(
+                                            "model requires weld-codegen version >= {}",
+                                            min_ver
+                                        )));
+                                    }
+                                } else {
+                                    return Err(Error::Model(
+                                        "missing 'min_version' in metadata.codegen for lang=rust"
+                                            .to_string(),
+                                    ));
+                                }
+                            }
+                        }
+                    }
+                }
+            }
             if let Some(packages) = model.metadata_value("package") {
                 let packages: Vec<PackageName> = serde_json::from_value(value_to_json(packages))
-                    .map_err(|e| Error::Model(format!("invalid metadata format for package, expecting format '[{{namespace:\"org.example\",crate:\"path::module\"}}]':  {}", e)))?;
+                    .map_err(|e| {
+                        Error::Model(format!(
+                            "invalid metadata format for package, expecting format \
+                             '[{{namespace:\"org.example\",crate:\"path::module\"}}]':  {}",
+                            e
+                        ))
+                    })?;
                 for p in packages.iter() {
                     self.packages.insert(p.namespace.to_string(), p.clone());
                 }
@@ -113,7 +172,6 @@ impl<'model> CodeGen for RustCodeGen<'model> {
 
     /// Perform any initialization required prior to code generation for a file
     /// `model` may be used to check model metadata
-    /// `id` is a tag from codegen.toml that indicates which source file is to be written
     /// `namespace` is the namespace in the model to generate
     #[allow(unused_variables)]
     fn init_file(
@@ -127,6 +185,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
             Some(ns) => Some(NamespaceID::from_str(ns)?),
             None => None,
         };
+        //self.protocol = model.wasmbus_
         if let Some(ref ns) = self.namespace {
             if self.packages.get(&ns.to_string()).is_none() {
                 print_warning(&format!(
@@ -150,28 +209,23 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         &mut self,
         w: &mut Writer,
         model: &Model,
-        _params: &ParamMap,
+        params: &ParamMap,
     ) -> Result<()> {
-        cfg_if::cfg_if! {
-            if #[cfg(feature = "cbor")] {
-                let import_cbor = "minicbor,";
-            }  else {
-                let import_cbor = "";
-            }
-        }
-        w.write(
-            r#"// This file is generated automatically using wasmcloud/weld-codegen and smithy model definitions
-               //
-            "#);
+        w.write(&format!(
+            "// This file is generated automatically using wasmcloud/weld-codegen {}\n",
+            env!("CARGO_PKG_VERSION")
+        ));
         match &self.namespace {
             Some(n) if n == wasmcloud_model_namespace() => {
                 // the base model has minimal dependencies
-                w.write(
-                    r#"
-                #![allow(dead_code, unused_imports, clippy::ptr_arg, clippy::needless_lifetimes)]
-                use serde::{{Deserialize, Serialize}};
-             "#,
-                );
+                w.write("#[allow(unused_imports)] use serde::{{Deserialize, Serialize}};\n");
+                w.write(b"#[allow(unused_imports)] use minicbor::{Encode,encode::Write};\n");
+                if !params.contains_key("no_serde") {
+                    w.write(&format!(
+                        "#[allow(unused_imports)] use {}::error::{{RpcError,RpcResult}};\n",
+                        self.import_core
+                    ));
+                }
             }
             _ => {
                 // all others use standard frontmatter
@@ -180,16 +234,24 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 // if the crate we are generating is "wasmbus_rpc" then we have to import it with "crate::".
                 w.write(&format!(
                     r#"
-                #![allow(unused_imports, clippy::ptr_arg, clippy::needless_lifetimes)]
+                #[allow(unused_imports)]
                 use {}::{{
-                    Context, deserialize, serialize, MessageDispatch, RpcError, RpcResult,
-                    Timestamp, Transport, Message, SendOpts, {}
+                    cbor::*,
+                    common::{{
+                        Context, deserialize, Message, MessageFormat, message_format, 
+                        MessageDispatch, SendOpts, serialize, Transport,
+                    }},
+                    error::{{RpcError,RpcResult}},
+                    Timestamp,
                 }};
+                #[allow(unused_imports)]
                 use serde::{{Deserialize, Serialize}};
+                #[allow(unused_imports)]
                 use async_trait::async_trait;
-                use std::{{borrow::Cow, io::Write, string::ToString}};
+                #[allow(unused_imports)]
+                use std::{{borrow::Borrow, borrow::Cow, io::Write, string::ToString}};
                 "#,
-                    &self.import_core, import_cbor,
+                    &self.import_core,
                 ));
             }
         }
@@ -200,7 +262,7 @@ impl<'model> CodeGen for RustCodeGen<'model> {
         Ok(())
     }
 
-    fn declare_types(&mut self, w: &mut Writer, model: &Model, _params: &ParamMap) -> Result<()> {
+    fn declare_types(&mut self, w: &mut Writer, model: &Model, params: &ParamMap) -> Result<()> {
         let ns = self.namespace.clone();
 
         let mut shapes = model
@@ -246,13 +308,19 @@ impl<'model> CodeGen for RustCodeGen<'model> {
                 | ShapeKind::Union(_)
                 | ShapeKind::Unresolved => {}
             }
-            cfg_if::cfg_if! {
-                if #[cfg(feature = "cbor")] {
-                    if !traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap()) {
-                        self.declare_shape_encoder(w, id, shape)?;
-                        self.declare_shape_decoder(w, id, shape)?;
-                    }
-                }
+
+            // If the shape is not a trait, and ser-deser isn't disabled, generate encoder and decoder
+            // It's ok to declare cbor shape encoders & decoders even if not used by
+            // this service's protocol version, because this shape might be used in other interfaces
+            // that _do_ use cbor, and because the cbor en-/de- coders won't be added
+            // to the compiler output of they aren't used
+            if !traits.contains_key(&prelude_shape_named(TRAIT_TRAIT).unwrap())
+                && !params.contains_key("no_serde")
+            //&& env!("CARGO_PKG_NAME") != "weld-codegen"
+            // wasmbus-rpc can't be as dependency of weld-codegen or it creates circular dependencies
+            {
+                self.declare_shape_encoder(w, id, shape)?;
+                self.declare_shape_decoder(w, id, shape)?;
             }
         }
         Ok(())
@@ -266,9 +334,14 @@ impl<'model> CodeGen for RustCodeGen<'model> {
             .map(|s| (s.id(), s.traits(), s.body()))
         {
             if let ShapeKind::Service(service) = shape {
-                self.write_service_interface(w, model, id.shape_name(), traits, service)?;
-                self.write_service_receiver(w, model, id.shape_name(), traits, service)?;
-                self.write_service_sender(w, model, id.shape_name(), traits, service)?;
+                let service = ServiceInfo {
+                    id: id.shape_name(),
+                    service,
+                    traits,
+                };
+                self.write_service_interface(w, model, &service)?;
+                self.write_service_receiver(w, model, &service)?;
+                self.write_service_sender(w, model, &service)?;
             }
         }
         Ok(())
@@ -292,7 +365,6 @@ impl<'model> CodeGen for RustCodeGen<'model> {
 }
 
 /// returns true if the file path ends in ".rs"
-#[allow(clippy::ptr_arg)]
 pub(crate) fn is_rust_source(path: &Path) -> bool {
     match path.extension() {
         Some(s) => s.to_string_lossy().as_ref() == "rs",
@@ -314,7 +386,7 @@ impl<'model> RustCodeGen<'model> {
             self.write_documentation(w, id, text);
         }
 
-        // deprecated
+        // '@deprecated' trait
         if let Some(Some(Value::Object(map))) =
             traits.get(&prelude_shape_named(TRAIT_DEPRECATED).unwrap())
         {
@@ -328,7 +400,7 @@ impl<'model> RustCodeGen<'model> {
             w.write(b")\n");
         }
 
-        // unstable
+        // '@unstable' trait
         if traits
             .get(&prelude_shape_named(TRAIT_UNSTABLE).unwrap())
             .is_some()
@@ -345,6 +417,7 @@ impl<'model> RustCodeGen<'model> {
             Ty::Shape(field.target())
         })
     }
+
     /// Write a type name, a primitive or defined type, with or without deref('&') and with or without Option<>
     pub(crate) fn type_string(&self, ty: Ty<'_>) -> Result<String> {
         let mut s = String::new();
@@ -427,8 +500,14 @@ impl<'model> RustCodeGen<'model> {
                             s.push_str(&self.to_type_name(&id.shape_name().to_string()));
                         }
                         _ => {
-                            return Err(Error::Model(format!("undefined crate for namespace {} for symbol {}. Make sure codegen.toml includes all dependent namespaces, and that the dependent .smithy file contains package metadata with crate: value",
-                                                            &id.namespace(), &id)));
+                            return Err(Error::Model(format!(
+                                "undefined crate for namespace {} for symbol {}. Make sure \
+                                 codegen.toml includes all dependent namespaces, and that the \
+                                 dependent .smithy file contains package metadata with crate: \
+                                 value",
+                                &id.namespace(),
+                                &id
+                            )));
                         }
                     }
                 }
@@ -606,9 +685,7 @@ impl<'model> RustCodeGen<'model> {
             // even if msgpack is not used for serialization, so it seems like
             // an acceptable simplification.
             #[cfg(feature = "wasmbus")]
-            if member.target() == &ShapeID::new_unchecked("smithy.api", "Blob", None)
-            //&& traits.get(wasmbus_data_trait()).is_some()
-            {
+            if member.target() == &ShapeID::new_unchecked("smithy.api", "Blob", None) {
                 w.write(r#"  #[serde(with="serde_bytes")] "#);
             }
 
@@ -649,28 +726,26 @@ impl<'model> RustCodeGen<'model> {
         &mut self,
         w: &mut Writer,
         model: &Model,
-        service_id: &Identifier,
-        service_traits: &AppliedTraits,
-        service: &Service,
+        service: &ServiceInfo,
     ) -> Result<()> {
-        self.apply_documentation_traits(w, service_id, service_traits);
+        self.apply_documentation_traits(w, service.id, service.traits);
 
         #[cfg(feature = "wasmbus")]
-        self.add_wasmbus_comments(w, service_id, service_traits)?;
+        self.add_wasmbus_comments(w, service)?;
 
         w.write(b"#[async_trait]\npub trait ");
-        self.write_ident(w, service_id);
+        self.write_ident(w, service.id);
         w.write(b"{\n");
-        self.write_service_contract_getter(w, service_id, service_traits)?;
+        self.write_service_contract_getter(w, service)?;
 
-        for operation in service.operations() {
+        for operation in service.service.operations() {
             // if operation is not declared in this namespace, don't define it here
             if let Some(ref ns) = self.namespace {
                 if operation.namespace() != ns {
                     continue;
                 }
             }
-            let (op, op_traits) = get_operation(model, operation, service_id)?;
+            let (op, op_traits) = get_operation(model, operation, service.id)?;
             let method_id = operation.shape_name();
             let _flags = self.write_method_signature(w, method_id, op_traits, op)?;
             w.write(b";\n");
@@ -683,14 +758,9 @@ impl<'model> RustCodeGen<'model> {
     fn write_service_contract_getter(
         &mut self,
         w: &mut Writer,
-        _service_id: &Identifier,
-        service_traits: &AppliedTraits,
+        service: &ServiceInfo,
     ) -> Result<()> {
-        if let Some(Wasmbus {
-            contract_id: Some(contract_id),
-            ..
-        }) = get_trait(service_traits, crate::model::wasmbus_trait())?
-        {
+        if let Some(contract_id) = service.wasmbus_contract_id() {
             w.write(&format!(
                 r#"
                 /// returns the capability contract id for this interface
@@ -703,26 +773,21 @@ impl<'model> RustCodeGen<'model> {
     }
 
     #[cfg(feature = "wasmbus")]
-    fn add_wasmbus_comments(
-        &mut self,
-        w: &mut Writer,
-        service_id: &Identifier,
-        service_traits: &AppliedTraits,
-    ) -> Result<()> {
+    fn add_wasmbus_comments(&mut self, w: &mut Writer, service: &ServiceInfo) -> Result<()> {
         // currently the only thing we do with Wasmbus in codegen is add comments
-        let wasmbus: Option<Wasmbus> = get_trait(service_traits, crate::model::wasmbus_trait())?;
+        let wasmbus: Option<Wasmbus> = get_trait(service.traits, crate::model::wasmbus_trait())?;
         if let Some(wasmbus) = wasmbus {
-            if let Some(contract) = wasmbus.contract_id {
-                let text = format!("wasmbus.contractId: {}", &contract);
-                self.write_documentation(w, service_id, &text);
+            if let Some(contract_id) = service.wasmbus_contract_id() {
+                let text = format!("wasmbus.contractId: {}", contract_id);
+                self.write_documentation(w, service.id, &text);
             }
             if wasmbus.provider_receive {
                 let text = "wasmbus.providerReceive";
-                self.write_documentation(w, service_id, text);
+                self.write_documentation(w, service.id, text);
             }
             if wasmbus.actor_receive {
                 let text = "wasmbus.actorReceive";
-                self.write_documentation(w, service_id, text);
+                self.write_documentation(w, service.id, text);
             }
         }
         Ok(())
@@ -772,20 +837,19 @@ impl<'model> RustCodeGen<'model> {
         &mut self,
         w: &mut Writer,
         model: &Model,
-        service_id: &Identifier,
-        service_traits: &AppliedTraits,
-        service: &Service,
+        service: &ServiceInfo,
     ) -> Result<()> {
         let doc = format!(
             "{}Receiver receives messages defined in the {} service trait",
-            service_id, service_id
+            service.id, service.id
         );
         self.write_comment(w, CommentKind::Documentation, &doc);
-        self.apply_documentation_traits(w, service_id, service_traits);
+        self.apply_documentation_traits(w, service.id, service.traits);
         w.write(b"#[doc(hidden)]\n#[async_trait]\npub trait ");
-        self.write_ident_with_suffix(w, service_id, "Receiver")?;
+        self.write_ident_with_suffix(w, service.id, "Receiver")?;
         w.write(b" : MessageDispatch + ");
-        self.write_ident(w, service_id);
+        self.write_ident(w, service.id);
+        let proto = crate::model::wasmbus_proto(service.traits)?;
         w.write(
             br#"{
             async fn dispatch(
@@ -796,7 +860,7 @@ impl<'model> RustCodeGen<'model> {
         "#,
         );
 
-        for method_id in service.operations() {
+        for method_id in service.service.operations() {
             // we don't add operations defined in another namespace
             if let Some(ref ns) = self.namespace {
                 if method_id.namespace() != ns {
@@ -804,25 +868,34 @@ impl<'model> RustCodeGen<'model> {
                 }
             }
             let method_ident = method_id.shape_name();
-            let (op, method_traits) = get_operation(model, method_id, service_id)?;
+            let (op, method_traits) = get_operation(model, method_id, service.id)?;
             w.write(b"\"");
             w.write(&self.op_dispatch_name(method_ident));
             w.write(b"\" => {\n");
             if let Some(op_input) = op.input() {
+                let symbol = op_input.shape_name().to_string();
                 // let value : InputType = deserialize(...)?;
-                #[cfg(feature = "cbor-msg")]
-                w.write(b"let d = &mut minicbor::Decoder::new(&message.arg);\n");
-                w.write(b"let value: ");
-                self.write_type(w, Ty::Shape(op_input))?;
-                w.write(b" = ");
-                cfg_if::cfg_if! {
-                    if #[cfg(feature = "cbor-msg")] {
-                        w.write(&self.decode_shape_id(op_input)?);
-                        w.write(";\n");
-                    } else {
-                        w.write(b" deserialize(message.arg.as_ref())\
-                            .map_err(|e| RpcError::Deser(format!(\"message '{}': {}\", message.method, e)))?;\n");
-                    }
+                if proto.has_cbor() {
+                    w.write(&format!(
+                        r#"
+                    let value : {} = {}::common::decode(&message.arg, &decode_{})
+                      .map_err(|e| RpcError::Deser(format!("'{}': {{}}", e)))?;
+                    "#,
+                        self.type_string(Ty::Shape(op_input))?,
+                        self.import_core,
+                        crate::strings::to_snake_case(&symbol),
+                        &symbol,
+                    ));
+                } else {
+                    w.write(&format!(
+                        r#"
+                        let value: {} = {}::common::deserialize(&message.arg)
+                      .map_err(|e| RpcError::Deser(format!("'{}': {{}}", e)))?;
+                        "#,
+                        self.type_string(Ty::Shape(op_input))?,
+                        self.import_core,
+                        &symbol,
+                    ))
                 }
             }
             // let resp = Trait::method(self, ctx, &value).await?;
@@ -832,7 +905,7 @@ impl<'model> RustCodeGen<'model> {
                 w.write(b"let _resp = ");
             }
             let method_name = self.to_method_name(method_ident, method_traits);
-            self.write_ident(w, service_id); // Service::method
+            self.write_ident(w, service.id); // Service::method
             w.write(b"::");
             w.write(&method_name);
             w.write(b"(self, ctx");
@@ -843,25 +916,34 @@ impl<'model> RustCodeGen<'model> {
 
             if let Some(_op_output) = op.output() {
                 // serialize result
-                cfg_if::cfg_if! {
-                    if #[cfg(feature = "cbor-msg")] {
-                        w.write(b"let mut buf = Vec::new(); let e = &mut minicbor::Encoder::new(&mut buf);\n");
-                        let s = self.encode_shape_id(_op_output, crate::encode_rust::ValExpr::Plain("resp"))?;
-                        w.write(&s);
-                    } else {
-                        w.write("let buf = serialize(&resp)?;\n");
-                    }
+                if proto.has_cbor() {
+                    w.write(&format!(
+                        "let mut e = {}::cbor::vec_encoder();\n",
+                        &self.import_core
+                    ));
+                    let s = self.encode_shape_id(
+                        _op_output,
+                        crate::encode_rust::ValExpr::Plain("resp"),
+                        true,
+                    )?;
+                    w.write(&s);
+                    w.write(b"let buf = e.into_inner();\n");
+                } else {
+                    w.write(&format!(
+                        "let buf = {}::common::serialize(&resp)?;\n",
+                        &self.import_core
+                    ));
                 }
             } else {
                 w.write(b"let buf = Vec::new();\n");
             }
             //w.write(br#"console_log(format!("actor result {}b",buf.len())); "#);
             w.write(b"Ok(Message { method: \"");
-            w.write(&self.full_dispatch_name(service_id, method_ident));
+            w.write(&self.full_dispatch_name(service.id, method_ident));
             w.write(b"\", arg: Cow::Owned(buf) })},\n");
         }
         w.write(b"_ => Err(RpcError::MethodNotHandled(format!(\"");
-        self.write_ident(w, service_id);
+        self.write_ident(w, service.id);
         w.write(b"::{}\", message.method))),\n");
         w.write(b"}\n}\n}\n\n"); // end match, end fn dispatch, end trait
 
@@ -874,16 +956,15 @@ impl<'model> RustCodeGen<'model> {
         &mut self,
         w: &mut Writer,
         model: &Model,
-        service_id: &Identifier,
-        service_traits: &AppliedTraits,
-        service: &Service,
+        service: &ServiceInfo,
     ) -> Result<()> {
         let doc = format!(
             "{}Sender sends messages to a {} service",
-            service_id, service_id
+            service.id, service.id
         );
         self.write_comment(w, CommentKind::Documentation, &doc);
-        self.apply_documentation_traits(w, service_id, service_traits);
+        self.apply_documentation_traits(w, service.id, service.traits);
+        let proto = crate::model::wasmbus_proto(service.traits)?;
         w.write(&format!(
             r#"/// client for sending {} messages
               #[derive(Debug)]
@@ -900,21 +981,21 @@ impl<'model> RustCodeGen<'model> {
                   }}
               }}
             "#,
-            service_id, service_id, service_id, service_id,
+            service.id, service.id, service.id, service.id,
         ));
         #[cfg(feature = "wasmbus")]
-        w.write(&self.actor_receive_sender_constructors(service_id, service_traits)?);
+        w.write(&self.actor_receive_sender_constructors(service.id, service.traits)?);
         #[cfg(feature = "wasmbus")]
-        w.write(&self.provider_receive_sender_constructors(service_id, service_traits)?);
+        w.write(&self.provider_receive_sender_constructors(service.id, service.traits)?);
 
         // implement Trait for TraitSender
         w.write(b"#[async_trait]\nimpl<T:Transport + std::marker::Sync + std::marker::Send> ");
-        self.write_ident(w, service_id);
+        self.write_ident(w, service.id);
         w.write(b" for ");
-        self.write_ident_with_suffix(w, service_id, "Sender")?;
+        self.write_ident_with_suffix(w, service.id, "Sender")?;
         w.write(b"<T> {\n");
 
-        for method_id in service.operations() {
+        for method_id in service.service.operations() {
             // we don't add operations defined in another namespace
             if let Some(ref ns) = self.namespace {
                 if method_id.namespace() != ns {
@@ -923,35 +1004,43 @@ impl<'model> RustCodeGen<'model> {
             }
             let method_ident = method_id.shape_name();
 
-            let (op, method_traits) = get_operation(model, method_id, service_id)?;
+            let (op, method_traits) = get_operation(model, method_id, service.id)?;
             w.write(b"#[allow(unused)]\n");
             let arg_flags = self.write_method_signature(w, method_ident, method_traits, op)?;
             let _arg_is_string = matches!(arg_flags, MethodArgFlags::ToString);
             w.write(b" {\n");
             if let Some(_op_input) = op.input() {
-                cfg_if::cfg_if! {
-                    if #[cfg(feature = "cbor-msg")] {
-                        if _arg_is_string {
-                            w.write(b"let arg = arg.to_string();\n");
-                        }
-                        w.write(b"let mut buf = Vec::new(); let e = &mut minicbor::Encoder::new(&mut buf);\n");
-                        let s = self.encode_shape_id(_op_input,
-                            if _arg_is_string {
-                                crate::encode_rust::ValExpr::Ref("arg.as_ref()")
-                            } else {
-                                crate::encode_rust::ValExpr::Ref("arg")
-                            }
-                        )?;
-                        w.write(&s);
-                        //let tn = crate::strings::to_snake_case(&self.type_string(Ty::Shape(op.input().as_ref().unwrap()))?);
-                        //w.write(&format!("encode_{}(&mut e, arg)?;", tn));
-                    } else {
-                        if matches!(arg_flags, MethodArgFlags::ToString) {
-                            w.write(b"let buf = serialize(&arg.to_string())?;\n");
-                        } else {
-                            w.write(b"let buf = serialize(arg)?;\n");
-                        }
+                if proto.has_cbor() {
+                    if _arg_is_string {
+                        w.write(b"let arg = arg.to_string();\n");
                     }
+                    w.write(&format!(
+                        "let mut e = {}::cbor::vec_encoder();\n",
+                        &self.import_core
+                    ));
+                    let s = self.encode_shape_id(
+                        _op_input,
+                        if _arg_is_string {
+                            crate::encode_rust::ValExpr::Ref("arg.as_ref()")
+                        } else {
+                            crate::encode_rust::ValExpr::Ref("arg")
+                        },
+                        true,
+                    )?;
+                    w.write(&s);
+                    w.write(b"let buf = e.into_inner(); \n");
+                    //let tn = crate::strings::to_snake_case(&self.type_string(Ty::Shape(op.input().as_ref().unwrap()))?);
+                    //w.write(&format!("encode_{}(&mut e, arg)?;", tn));
+                } else if matches!(arg_flags, MethodArgFlags::ToString) {
+                    w.write(&format!(
+                        "let buf = {}::common::serialize(&arg.to_string())?;\n",
+                        &self.import_core
+                    ));
+                } else {
+                    w.write(&format!(
+                        "let buf = {}::common::serialize(arg)?;\n",
+                        &self.import_core
+                    ));
                 }
             } else {
                 w.write(b"let buf = *b\"\";\n");
@@ -959,24 +1048,35 @@ impl<'model> RustCodeGen<'model> {
             w.write(b"let resp = self.transport.send(ctx, Message{ method: ");
             // note: legacy is just the latter part
             w.write(b"\"");
-            w.write(&self.full_dispatch_name(service_id, method_ident));
+            w.write(&self.full_dispatch_name(service.id, method_ident));
             //w.write(&self.op_dispatch_name(method_ident));
             w.write(b"\", arg: Cow::Borrowed(&buf)}, None).await?;\n");
-            if let Some(_op_output) = op.output() {
-                cfg_if::cfg_if! {
-                    if #[cfg(feature = "cbor-msg")] {
-                        w.write(b"let d = &mut minicbor::Decoder::new(&resp);\n");
-                        w.write(b"let value = ");
-                        w.write(&self.decode_shape_id(_op_output)?);
-                        w.write(b";");
-                    } else {
-                        w.write(b"let value = deserialize(&resp)");
-                        w.write(b".map_err(|e| RpcError::Deser(format!(\"response to {}: {}\", \"");
-                        w.write(&method_ident.to_string());
-                        w.write(b"\", e)))?;");
-                    }
+            if let Some(op_output) = op.output() {
+                let symbol = op_output.shape_name().to_string();
+                if proto.has_cbor() {
+                    w.write(&format!(
+                        r#"
+                    let value : {} = {}::common::decode(&resp, &decode_{})
+                        .map_err(|e| RpcError::Deser(format!("'{{}}': {}", e)))?;
+                    Ok(value)
+                    "#,
+                        self.type_string(Ty::Shape(op_output))?,
+                        self.import_core,
+                        crate::strings::to_snake_case(&symbol),
+                        &symbol,
+                    ));
+                } else {
+                    w.write(&format!(
+                        r#"
+                    let value : {} = {}::common::deserialize(&resp)
+                        .map_err(|e| RpcError::Deser(format!("'{{}}': {}", e)))?;
+                    Ok(value)
+                    "#,
+                        self.type_string(Ty::Shape(op_output))?,
+                        self.import_core,
+                        &symbol,
+                    ));
                 }
-                w.write(b"Ok(value)");
             } else {
                 w.write(b"Ok(())");
             }
@@ -1116,3 +1216,15 @@ Opt   @required   @box    bool/int/...
 x     1           1       0
 x     1           1       1
 */
+
+// check that the codegen package has a parseable version
+#[test]
+fn package_semver() {
+    let package_version = env!("CARGO_PKG_VERSION");
+    let version = semver::Version::parse(package_version);
+    assert!(
+        version.is_ok(),
+        "package version {} has unexpected format",
+        package_version
+    );
+}
