@@ -1,31 +1,24 @@
-#![cfg(feature = "cbor")]
 // CBOR Encode functions
 //
 // Because we have all the type information for declared types,
 // we can invoke the appropriate encode_* functions for each
 // simple type, structure, map, and array. (later: enums).
-// If we had leveraged minicbor's Encoder trait , we could have let
-// the rust compiler do the work to invoke nested encoders for complex types.
-// Instead, we generate encode_* functions for each non-simple data type
-// and call them directly. It is hoped that this will simplify ports to other
-// target languages, if those languages don't have traits, and the cbor libraries
-// in those languages can't #[derive] encoders and decoders for arbitrary
-// structures.
+// For Rust, it would have been a little easier to use serde code generation,
+// but we want to create a code base that is easy to port to other languages
+// that don't have serde.
 
 // The encoder is written as a plain function "encode_<S>" where S is the type name
 // (camel cased for the fn name), and scoped to the module where S is defined.
-use crate::codegen_rust::RustCodeGen;
 use crate::{
-    codegen_rust::is_optional_type,
+    codegen_rust::{is_optional_type, RustCodeGen},
     error::{Error, Result},
     gen::CodeGen,
     model::wasmcloud_model_namespace,
     writer::Writer,
 };
-use atelier_core::model::shapes::ShapeKind;
 use atelier_core::{
     model::{
-        shapes::{HasTraits, Simple, StructureOrUnion},
+        shapes::{HasTraits, ShapeKind, Simple, StructureOrUnion},
         HasIdentity, ShapeID,
     },
     prelude::{
@@ -129,7 +122,12 @@ impl<'model> RustCodeGen<'model> {
     /// Generates cbor encode statements "e.func()" for the id.
     /// If id is a primitive type, writes the direct encode function, otherwise,
     /// delegates to an encode_* function created in the same module where the symbol is defined
-    pub(crate) fn encode_shape_id(&self, id: &ShapeID, val: ValExpr) -> Result<String> {
+    pub(crate) fn encode_shape_id(
+        &self,
+        id: &ShapeID,
+        val: ValExpr,
+        enc_owned: bool, // true if encoder is owned in current fn
+    ) -> Result<String> {
         let name = id.shape_name().to_string();
         let stmt = if id.namespace() == prelude_namespace_id() {
             match name.as_ref() {
@@ -167,7 +165,7 @@ impl<'model> RustCodeGen<'model> {
                         s.push_str("::model::");
                     }
                     s.push_str(&format!(
-                        "encode_{}(e, {})?;\n",
+                        "encode_{}( e, {})?;\n",
                         crate::strings::to_snake_case(&id.shape_name().to_string()),
                         val.as_ref()
                     ));
@@ -176,8 +174,9 @@ impl<'model> RustCodeGen<'model> {
             }
         } else if self.namespace.is_some() && id.namespace() == self.namespace.as_ref().unwrap() {
             format!(
-                "encode_{}(e, {})?;\n",
+                "encode_{}({} e, {})?;\n",
                 crate::strings::to_snake_case(&id.shape_name().to_string()),
+                if enc_owned { "&mut " } else { "" },
                 val.as_ref()
             )
         } else {
@@ -188,15 +187,21 @@ impl<'model> RustCodeGen<'model> {
                 }) => {
                     // the crate name should be valid rust syntax. If not, they'll get an error with rustc
                     format!(
-                        "{}::encode_{}(e, {})?;\n",
+                        "{}::encode_{}( {} e, {})?;\n",
                         &crate_name,
                         crate::strings::to_snake_case(&id.shape_name().to_string()),
+                        if enc_owned { "&mut " } else { "" },
                         val.as_ref(),
                     )
                 }
                 _ => {
-                    return Err(Error::Model(format!("undefined crate for namespace {} for symbol {}. Make sure codegen.toml includes all dependent namespaces, and that the dependent .smithy file contains package metadata with crate: value",
-                                                    &id.namespace(), &id)));
+                    return Err(Error::Model(format!(
+                        "undefined crate for namespace {} for symbol {}. Make sure codegen.toml \
+                         includes all dependent namespaces, and that the dependent .smithy file \
+                         contains package metadata with crate: value",
+                        &id.namespace(),
+                        &id
+                    )));
                 }
             }
         };
@@ -237,8 +242,12 @@ impl<'model> RustCodeGen<'model> {
                     val.as_str(),
                     val.as_str()
                 );
-                s.push_str(&self.encode_shape_id(map.key().target(), ValExpr::Ref("k"))?);
-                s.push_str(&self.encode_shape_id(map.value().target(), ValExpr::Ref("v"))?);
+                s.push_str(&self.encode_shape_id(map.key().target(), ValExpr::Ref("k"), false)?);
+                s.push_str(&self.encode_shape_id(
+                    map.value().target(),
+                    ValExpr::Ref("v"),
+                    false,
+                )?);
                 s.push_str(
                     r#"
                     }
@@ -255,7 +264,11 @@ impl<'model> RustCodeGen<'model> {
                     val.as_str(),
                     val.as_str()
                 );
-                s.push_str(&self.encode_shape_id(list.member().target(), ValExpr::Ref("item"))?);
+                s.push_str(&self.encode_shape_id(
+                    list.member().target(),
+                    ValExpr::Ref("item"),
+                    false,
+                )?);
                 s.push_str(
                     r#"
                     }
@@ -272,7 +285,11 @@ impl<'model> RustCodeGen<'model> {
                     val.as_str(),
                     val.as_str()
                 );
-                s.push_str(&self.encode_shape_id(set.member().target(), ValExpr::Ref("v"))?);
+                s.push_str(&self.encode_shape_id(
+                    set.member().target(),
+                    ValExpr::Ref("v"),
+                    false,
+                )?);
                 s.push_str(
                     r#"
                     }
@@ -330,7 +347,7 @@ impl<'model> RustCodeGen<'model> {
                 }
             }
             let field_name = self.to_field_name(field.id(), field.traits())?;
-            let field_val = self.encode_shape_id(field.target(), ValExpr::Ref("val"))?;
+            let field_val = self.encode_shape_id(field.target(), ValExpr::Ref("val"), false)?;
             if is_optional_type(field) {
                 s.push_str(&format!(
                     "if let Some(val) =  {}.{}.as_ref() {{\n",
@@ -349,7 +366,7 @@ impl<'model> RustCodeGen<'model> {
                     s.push_str(&format!("e.str(\"{}\")?;\n", field.id()));
                 }
                 let val = format!("{}.{}", val.as_str(), &field_name);
-                s.push_str(&self.encode_shape_id(field.target(), ValExpr::Plain(&val))?);
+                s.push_str(&self.encode_shape_id(field.target(), ValExpr::Plain(&val), false)?);
             }
             current_index += 1;
         }
@@ -385,18 +402,17 @@ impl<'model> RustCodeGen<'model> {
                 let mut s = format!(
                     r#" 
                 // Encode {} as CBOR and append to output stream
-                // This is part of experimental cbor support
                 #[doc(hidden)] {}
-                pub fn encode_{}<W>(e: &mut minicbor::Encoder<W>, {}: &{}) -> Result<(),minicbor::encode::Error<W::Error>>
-                where
-                    W: minicbor::encode::Write,
+                pub fn encode_{}<W: {}::cbor::Write>(e: &mut {}::cbor::Encoder<W>, {}: &{}) -> RpcResult<()>
                 {{
                 "#,
                     &name,
                     if is_rust_copy { "#[inline]" } else { "" },
                     crate::strings::to_snake_case(&name.to_string()),
+                    self.import_core,
+                    self.import_core,
                     if is_empty_struct { "_val" } else { "val" },
-                    &id.shape_name()
+                    &id.shape_name(),
                 );
                 s.push_str(&body);
                 s.push_str("Ok(())\n}\n");
