@@ -1,16 +1,13 @@
 #![forbid(unsafe_code)]
 
-mod error;
-pub use error::{Error, Result};
-#[cfg(feature = "cbor")]
+pub mod error;
+use error::Error;
 pub(crate) mod codegen_py;
 pub(crate) mod codegen_rust;
 pub mod config;
-#[cfg(feature = "cbor")]
 pub(crate) mod decode_py;
 pub(crate) mod decode_rust;
 pub mod docgen;
-#[cfg(feature = "cbor")]
 pub(crate) mod encode_py;
 pub(crate) mod encode_rust;
 pub(crate) mod gen;
@@ -18,17 +15,19 @@ mod loader;
 pub(crate) mod model;
 pub mod render;
 pub mod writer;
-pub use gen::templates_from_dir;
-pub use gen::Generator;
+pub use gen::{templates_from_dir, Generator};
 
 pub(crate) use loader::sources_to_paths;
 pub use loader::{sources_to_model, weld_cache_dir};
-pub use rust_build::rust_build;
+pub use rust_build::{rust_build, rust_build_into};
 
 pub mod format;
 mod rust_build;
 
-pub(crate) mod wasmbus_model;
+#[allow(dead_code)]
+pub(crate) mod wasmbus_model {
+    include!("./wasmbus_model.rs");
+}
 
 // re-export
 pub use bytes::Bytes;
