@@ -3,16 +3,13 @@
 //! - ModelIndex is a "cache" of a smithy model grouped by shape kind and sorted by identifier name
 //! - various macros used in the codegen crate
 //!
-use std::ops::Deref;
-//use crate::strings::{to_pascal_case, to_snake_case};
 use crate::{
     error::{Error, Result},
     JsonValue,
 };
-use atelier_core::model::shapes::StructureOrUnion;
 use atelier_core::{
     model::{
-        shapes::{AppliedTraits, HasTraits, MemberShape, Operation, ShapeKind},
+        shapes::{AppliedTraits, HasTraits, MemberShape, Operation, ShapeKind, StructureOrUnion},
         values::{Number, Value as NodeValue},
         HasIdentity, Identifier, Model, NamespaceID, ShapeID,
     },
@@ -20,7 +17,7 @@ use atelier_core::{
 };
 use lazy_static::lazy_static;
 use serde::{de::DeserializeOwned, Deserialize};
-use std::str::FromStr;
+use std::{fmt, ops::Deref, str::FromStr};
 
 const WASMCLOUD_MODEL_NAMESPACE: &str = "org.wasmcloud.model";
 const WASMCLOUD_CORE_NAMESPACE: &str = "org.wasmcloud.core";
@@ -119,6 +116,43 @@ pub enum CommentKind {
     InQuote,
 }
 
+#[derive(Default, Clone, PartialEq)]
+pub struct WasmbusProtoVersion {
+    base: u8, // base number
+}
+
+impl TryFrom<&str> for WasmbusProtoVersion {
+    type Error = crate::error::Error;
+
+    fn try_from(value: &str) -> std::result::Result<Self, Self::Error> {
+        match value {
+            "0" => Ok(WasmbusProtoVersion { base: 0 }),
+            "2" => Ok(WasmbusProtoVersion { base: 2 }),
+            _ => Err(Error::Model(format!(
+                "Invalid wasmbus.protocol: '{}'. The default value is \"0\".",
+                value
+            ))),
+        }
+    }
+}
+impl fmt::Debug for WasmbusProtoVersion {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        f.write_str(&self.to_string())
+    }
+}
+
+impl ToString for WasmbusProtoVersion {
+    fn to_string(&self) -> String {
+        format!("{}", self.base)
+    }
+}
+
+impl WasmbusProtoVersion {
+    pub fn has_cbor(&self) -> bool {
+        self.base >= 2
+    }
+}
+
 // Modifiers on data type
 // This enum may be extended in the future if other variations are required.
 // It's recursively composable, so you could represent &Option<&Value>
@@ -214,6 +248,16 @@ pub fn get_trait<T: DeserializeOwned>(traits: &AppliedTraits, id: &ShapeID) -> R
     }
 }
 
+pub fn wasmbus_proto(traits: &AppliedTraits) -> Result<WasmbusProtoVersion> {
+    match get_trait(traits, wasmbus_trait()) {
+        Ok(Some(Wasmbus {
+            protocol: Some(version),
+            ..
+        })) => WasmbusProtoVersion::try_from(version.as_str()),
+        _ => Ok(WasmbusProtoVersion::default()),
+    }
+}
+
 /// Convert trait object to its native type
 pub fn trait_value<T: DeserializeOwned>(value: &NodeValue) -> Result<T> {
     let json = value_to_json(value);
@@ -269,9 +313,9 @@ pub fn has_default(model: &'_ Model, member: &MemberShape) -> bool {
     let id = resolve(model, member.target());
     #[allow(unused_mut)]
     let mut has = false;
+    let name = id.shape_name().to_string();
 
     if id.namespace().eq(prelude_namespace_id()) {
-        let name = id.shape_name().to_string();
         cfg_if::cfg_if! {
             if #[cfg(feature = "BigInteger")] {
                 has = has || &name == "bigInteger";
@@ -292,6 +336,11 @@ pub fn has_default(model: &'_ Model, member: &MemberShape) -> bool {
             | "Timestamp"
         )
         // excluded: Resource, Operation, Service, Document, Union
+    } else if id.namespace() == wasmcloud_model_namespace() {
+        matches!(
+            name.as_str(),
+            "U64" | "U32" | "U16" | "U8" | "I64" | "I32" | "I16" | "I8"
+        )
     } else {
         false
         // for any other type, return false.
@@ -318,6 +367,7 @@ impl NumberedMember {
             })?,
         })
     }
+
     pub(crate) fn field_num(&self) -> &Option<u16> {
         &self.field_num
     }
@@ -331,7 +381,9 @@ impl Deref for NumberedMember {
     }
 }
 
+use crate::wasmbus_model::Wasmbus;
 use std::iter::Iterator;
+
 /// Returns sorted list of fields for the structure, and whether it is numbered.
 /// If there are any errors in numbering, returns Error::Model
 pub(crate) fn get_sorted_fields(
@@ -370,9 +422,11 @@ pub(crate) fn has_field_numbers(fields: &[NumberedMember], name: &str) -> Result
         // all fields are numbered uniquely
         Ok(true)
     } else {
-        Err(crate::Error::Model(
-                format!("structure {} has incomplete or invalid field numbers: either some fields are missing the '@n()' trait, or some fields have duplicate numbers.", name)
-            ))
+        Err(crate::Error::Model(format!(
+            "structure {} has incomplete or invalid field numbers: either some fields are missing \
+             the '@n()' trait, or some fields have duplicate numbers.",
+            name
+        )))
     }
 }
 
